/*
 * q_ingress.c             INGRESS.
 *
 *              This program is free software; you can redistribute it and/or
 *              modify it under the terms of the GNU General Public License
 *              as published by the Free Software Foundation; either version
 *              2 of the License, or (at your option) any later version.
 *
 * Authors:    J Hadi Salim
 */

#include <stdio.h>
#include <string.h>

#include "utils.h"
#include "tc_util.h"

static void explain(void)
{
	fprintf(stderr, "Usage: ... ingress [cacheable]\n");
}

static int ingress_parse_opt(struct qdisc_util *qu, int argc, char **argv,
			     struct nlmsghdr *n, const char *dev)
{
	struct rtattr *tail;
	__u32 flags = 0;

	while (argc > 0) {
		if (strcmp(*argv, "handle") == 0) {
			NEXT_ARG();
			argc--; argv++;
		} else if (strcmp(*argv, "cacheable") == 0) {
			flags |= TCA_INGRESS_ATTR_FLAG_CACHEABLE;
			NEXT_ARG_FWD();
		} else {
			fprintf(stderr, "What is \"%s\"?\n", *argv);
			explain();
			return -1;
		}
	}

	tail = addattr_nest(n, 1024, TCA_OPTIONS);

	if (flags)
		addattr_l(n, 1024, TCA_INGRESS_ATTR_FLAGS, &flags, sizeof(flags));

	addattr_nest_end(n, tail);

	return 0;
}

static int ingress_print_opt(struct qdisc_util *qu, FILE *f,
			     struct rtattr *opt)
{
	struct rtattr *tb[TCA_INGRESS_ATTR_MAX + 1];

	if (opt == NULL)
		return 0;

	parse_rtattr_nested(tb, TCA_INGRESS_ATTR_MAX, opt);

	if (tb[TCA_INGRESS_ATTR_FLAGS]) {
		__u32 flags;

		flags = rta_getattr_u32(tb[TCA_INGRESS_ATTR_FLAGS]);
		if (flags & TCA_INGRESS_ATTR_FLAG_CACHEABLE)
			print_string(PRINT_ANY, "flag", "%s", "cacheable");
	}

	return 0;
}

struct qdisc_util ingress_qdisc_util = {
	.id		= "ingress",
	.parse_qopt	= ingress_parse_opt,
	.print_qopt	= ingress_print_opt,
};
